PROGRAM YPDF
  USE cosmo
  USE linearpk
  USE sigma
  USE growth
  USE angular_distance
  USE fx
! This code calculates a bin-integrated one-point PDF of Compton Y from 
! massive halos that do not overlap in the sky. Dividing the bin-integrated
! PDF by the bin size, e.g., dY=1e-6, then gives a bin-averaged PDF.
! The original methodology is described in Hill et al., arXiv:1411.8004. 
! The actual implementation of this code is described in Klaus, Komatsu & 
! Sunyaev, arXiv:1509.05134. The output, "y_pdf.txt", contains:
! 1st: Compton Y parameter
! 2nd: PDF, integrated from the Y value in the 1st column to Y+dY with
! dT(=1e-6) given in the code.
! May 18, 2016: E.Komatsu
  IMPLICIT none
  double precision :: z1=1d-5,z2=5d0 ! redshift limits
  !!$ double precision :: z1=0.005d0,z2=3d0 ! redshift limits used by Hill et al.
  character(len=128) :: filename
  integer :: npk,nfx,i,j,n=1001
  double precision :: pdf,dPdlnx,lnx,dlnx,lnx1,lnx2
  double precision :: F_clust,Y
  double precision :: dY=1d-6 ! the width of Y over which the PDF is integrated
  external dPdlnx
! Specify cosmological parameters
! The data type has been defined in MODULE cosmo.
  ! Planck 2015 Parameters (with CMB lensing)
  !!$ om0=0.308d0
  !!$ ode0=0.692d0
  !!$ h0=0.6781d0
  !!$ obh2=0.02226d0
  !!$ w=-1d0
  ! WMAP9 Parameters
  !!$ om0=0.282d0
  !!$ ode0=0.718d0
  !!$ h0=0.697d0
  !!$ obh2=0.02240d0
  !!$ w=-1d0
  ! Magneticum Parameters
  om0=0.272d0
  ode0=0.728d0
  h0=0.704d0
  obh2=0.0456*h0**2d0
  w=-1d0
! read in and tabulate P(k)
  ! Planck 2015 Parameters (with CMB lensing)
  !!$ filename='planck15_matterpower.dat'
  !!$ npk=707 ! # of lines in the file
  ! WMAP9 Parameters
  !!$ filename='wmap9_matterpower.dat'
  !!$ npk=706 ! # of lines in the file
  ! Magneticum Parameters
  filename='wavenumber_pk_at_z=0_magneticum.txt'
  npk=896 ! # of lines in the file
  CALL open_linearpk(filename,npk)
! fit sigma^2(R) to Chebyshev polynomials
  CALL compute_sigma2
  CALL close_linearpk
! compute the growth factor
  CALL setup_growth
! compute and tabulate da(z) 
  CALL setup_da
! read in and tabulate f(x)
  filename='x_fx.txt'
  nfx=10000
  CALL open_file(filename,nfx)
! compute the mean Compton Y
  open(1,file='y_pdf.txt')
  lnx1=dlog(1d0+z1)
  lnx2=dlog(1d0+z2)
  dlnx=(lnx2-lnx1)/dble(n-1)
  pdf=dPdlnx(lnx1,1d-10)*(0.5d0*dlnx)
  pdf=pdf+dPdlnx(lnx2,1d-10)*(0.5d0*dlnx)
  do j=2,n-1
     lnx=lnx1+dble(j-1)*dlnx
     pdf=pdf+dPdlnx(lnx,1d-10)*dlnx
  enddo
  F_clust=pdf
  print*,'Overlapping fraction (F_clust)=',pdf
  print*,'Note: the calculation makes sense only if F_clust < 1'
  do i=1,201
     Y=1d-10+dble(i-1)*dY
     ! integrate by the trapezoidal rule
     pdf=(dPdlnx(lnx1,Y)-dPdlnx(lnx1,Y+dY))*(0.5d0*dlnx)
     pdf=pdf+(dPdlnx(lnx2,Y)-dPdlnx(lnx2,Y+dY))*(0.5d0*dlnx)
     do j=2,n-1
        lnx=lnx1+dble(j-1)*dlnx
        pdf=pdf+(dPdlnx(lnx,Y)-dPdlnx(lnx,Y+dY))*dlnx
     enddo
     if(i==1)then
        pdf=pdf+(1d0-F_clust) ! to normalise the PDF such that sum_i PDF(Y_i)=1
        Y=0d0
     endif
     write(1,'(2E15.5)')Y,pdf
  enddo
  close(1)
END PROGRAM YPDF
!--------------------------------------------------------------
double precision function dPdlnx(lnx,y)
  ! dPdlnx = (1+z) (dV/dz) int dlnM dn/dlnM pi theta^2
  ! lnx=ln(1+z)
  IMPLICIT none
  double precision, intent(IN) :: lnx,y
  double precision :: M1=2d14,M2=5d15 ! mass limits, h^-1 Msun
  double precision :: integrand,lnM1,lnM2,lnM,z
  external integrand
  z=exp(lnx)-1d0
  lnM1=dlog(M1) ! minimum mass, h^-1 Msun
  lnM2=dlog(M2) ! maximum mass, h^-1 Msun
  CALL qgaus2(integrand,lnM1,lnM2,dPdlnx,z,y) ! Gaussian quadrature
  dPdlnx=dPdlnx*(1d0+z)
  return
end function dPdlnx
